import axios from "axios";
import { defineAsyncComponent, nextTick } from "vue";
import router from "@/router";
/**
 * @description 防抖
 * @param func    功能函数(即要防抖的函数)
 * @param delay   延时时间
 * @param thisArg 功能函数内this的对象
 */
export function debounce(func: Function, delay: number, thisArg?: any) {
  let timer: number;
  return function (...args: any[]) {
    if (timer) {
      window.clearTimeout(timer);
    }
    timer = window.setTimeout(function () {
      func.apply(thisArg, args);
    }, delay);
  };
}
/**
 * @description 节流（原理同上）
 * @param func    功能函数(即要节流的函数)
 * @param delay   延时时间
 * @param thisArg 功能函数内this的对象
 */
export function throttle(func: Function, delay: number, thisArg?: any) {
  let timeStamp: number;
  return function (...args: any[]) {
    const nowTimeStamp = Date.now();
    if (!timeStamp || nowTimeStamp - timeStamp >= delay) {
      func.apply(thisArg, args);
      timeStamp = nowTimeStamp;
    }
  };
}
/**
 *  @description vite引入图片
 * @param {string} name   ../assets/images 后面的文件路径+文件名后缀
 * @returns {string} 返回处理后的文件地址
 */
export function getImageUrl(name: string) {
  return new URL(`../assets/img/${name}`, import.meta.url).href;
}
/**
 * @description 生成guid
 * @returns {string} 返回guid
 */
export function guid(): string {
  const S4 = (): string => {
    return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
  };
  return (
    S4() +
    S4() +
    "-" +
    S4() +
    "-" +
    S4() +
    "-" +
    S4() +
    "-" +
    S4() +
    S4() +
    S4()
  );
}
/**
 * 根据id排除自己（树形结构选择 修改时用）
 * @param {any} arr 需要排除的数组
 * @param {string} id 当前id
 * @returns {arr} 返回排除后的数组
 */
export function excludeSelfById(arr: Array<any>, id: string) {
  if (!arr) return [];
  return arr.filter((v: any) => {
    if (v.id === id) return false;
    if (v.children && v.children.length) {
      v.children = excludeSelfById(v.children, id);
    }
    return true;
  });
}
/**
 * @description 数组排序
 * @param arr 需要排序的数组
 * @param  key 排序的key
 * @param type 排序类型 默认降序 descending  升序 ascending
 * @returns {arr} 返回排序后的数组
 */
export function arraySortByKey(
  arr: Array<any>,
  key: string,
  type = "descending"
) {
  return arr.sort((a: any, b: any) => {
    if (type === "descending") {
      // 降序
      return b[key] - a[key];
    } else {
      // 升序
      return a[key] - b[key];
    }
  });
}

/**
 * @description 设置浏览器标题
 * @method const title = useTitle(); ==> title()
 */
export function useTitle() {
  nextTick(() => {
    let webTitle: string;
    const { meta } = router.currentRoute.value;
    webTitle = <any>meta.title || "市民热线服务系统";
    document.title = `${webTitle}`;
  }).then(() => {});
}
